@@ -1,4 +1,4 @@
-/* $OpenBSD: ocsp.c,v 1.29 2014/07/12 19:31:21 jsing Exp $ */
+/* $OpenBSD: ocsp.c,v 1.31 2014/08/24 16:07:29 bcook Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project 2000.
  */
@@ -57,6 +57,8 @@
  */
 #ifndef OPENSSL_NO_OCSP
 
+#include <sys/select.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <limits.h>
