@@ -1,4 +1,4 @@
-/* $OpenBSD: ts.c,v 1.17 2014/06/12 15:49:27 deraadt Exp $ */
+/* $OpenBSD: ts.c,v 1.1 2014/08/26 17:47:25 jsing Exp $ */
 /* Written by Zoltan Glozik (zglozik@stones.com) for the OpenSSL
  * project 2002.
  */
@@ -154,8 +154,6 @@ ts_main(int argc, char **argv)
 	/* Output is ContentInfo instead of TimeStampResp. */
 	int token_out = 0;
 
-	ERR_load_crypto_strings();
-
 	for (argc--, argv++; argc > 0; argc--, argv++) {
 		if (strcmp(*argv, "-config") == 0) {
 			if (argc-- < 1)
