@@ -1,4 +1,4 @@
-/* $OpenBSD: eng_rsax.c,v 1.9 2014/07/10 22:45:57 jsing Exp $ */
+/* $OpenBSD: eng_rsax.c,v 1.10 2014/07/12 16:03:37 miod Exp $ */
 /* Copyright (c) 2010-2010 Intel Corp.
  *   Author: Vinodh.Gopal@intel.com
  *           Jim Guilford
@@ -81,8 +81,9 @@
 /* RSAX is available **ONLY* on x86_64 CPUs */
 #undef COMPILE_RSAX
 
-#if (defined(__x86_64) || defined(__x86_64__) || \
-     defined(_M_AMD64) || defined (_M_X64)) && !defined(OPENSSL_NO_ASM)
+#if !defined(OPENSSL_NO_ASM) && defined(RSA_ASM) && \
+	(defined(__x86_64) || defined(__x86_64__) || \
+     defined(_M_AMD64) || defined (_M_X64))
 #define COMPILE_RSAX
 static ENGINE *ENGINE_rsax (void);
 #endif
